@@ -18,6 +18,7 @@ from sagemaker.feature_store.inputs import FeatureValue
 from pyspark.sql import DataFrame as SparkDataFrame
 from concurrent.futures import ThreadPoolExecutor
 from concurrent.futures import as_completed
+import concurrent.futures
 from typing import Union, List, Dict
 import pandas as pd
 import os
@@ -491,6 +492,7 @@ class SagemakerFeatureStore:
         self,
         feature_group_name: str,
         paritioned_spark_df: SparkDataFrame,
+        thread_num: int = 10,
     ):
         """
         Ingests a partitioned Spark DataFrame into the Feature Group.
@@ -509,31 +511,52 @@ class SagemakerFeatureStore:
         None
         """
 
-        def ingest_parallelize_spark_df(feature_group_name, rows):
-            rows = list(rows)
-            region = "ap-southeast-1"
-            os.environ["AWS_DEFAULT_REGION"] = region
-            session = boto3.session.Session()
-            runtime = session.client(
-                service_name="sagemaker-featurestore-runtime",
-                config=Config(retries={"max_attempts": 10, "mode": "standard"}),
-            )
-            for _, row in enumerate(rows):
-                record = [
-                    {"FeatureName": column, "ValueAsString": str(row[column])}
-                    for column in row.__fields__
-                ]
-                resp = runtime.put_record(
-                    FeatureGroupName=feature_group_name, Record=record
+        def ingest_parallelize_spark_df_multithread(
+            feature_group_name, rows, thread_num=10
+        ):
+            def ingest_worker(rows, feature_group_name):
+                region = "ap-southeast-1"
+                os.environ["AWS_DEFAULT_REGION"] = region
+                session = boto3.session.Session()
+                runtime = session.client(
+                    service_name="sagemaker-featurestore-runtime",
+                    config=Config(retries={"max_attempts": 10, "mode": "standard"}),
                 )
-                if not resp["ResponseMetadata"]["HTTPStatusCode"] == 200:
-                    raise (f"PutRecord failed: {resp}")
-            return
+                for row in rows:
+                    record = [
+                        {"FeatureName": column, "ValueAsString": str(row[column])}
+                        for column in row.__fields__
+                    ]
+                    resp = runtime.put_record(
+                        FeatureGroupName=feature_group_name, Record=record
+                    )
+                    if not resp["ResponseMetadata"]["HTTPStatusCode"] == 200:
+                        raise (f"PutRecord failed: {resp}")
+
+            rows = list(rows)
+            chunk_size = int(len(rows) / thread_num) + 1
+            chunked_rows_list = [
+                rows[i : i + chunk_size] for i in range(0, len(rows), chunk_size)
+            ]
+            with concurrent.futures.ThreadPoolExecutor(
+                max_workers=thread_num
+            ) as executor:
+                futures = [
+                    executor.submit(ingest_worker, chunked_rows, feature_group_name)
+                    for chunked_rows in chunked_rows_list
+                ]
+                for future in concurrent.futures.as_completed(futures):
+                    if future.exception() is not None:
+                        raise Exception(future.exception())
+                    else:
+                        pass
 
         print(f'Ingesting Spark Dataframe to "{feature_group_name}" Feature Store..')
         try:
             paritioned_spark_df.foreachPartition(
-                lambda rows: ingest_parallelize_spark_df(feature_group_name, rows)
+                lambda rows: ingest_parallelize_spark_df_multithread(
+                    feature_group_name, rows, thread_num
+                )
             )
         except Exception as e:
             raise Exception("ERROR:", e)
